@@ -19,35 +19,53 @@
 #  DEALINGS IN THE SOFTWARE.
 """Base Command Module."""
 
+from __future__ import annotations
+
 import abc
 import collections
-import glob
+import contextlib
 import logging
 import os
 import shutil
-from typing import Any, Callable
+import subprocess
+
+from typing import TYPE_CHECKING, Any
 
 import click
+import wcmatch.pathlib
+import wcmatch.wcmatch
+
 from click_help_colors import HelpColorsCommand, HelpColorsGroup
+from wcmatch import glob
 
 import molecule.scenarios
+
 from molecule import config, logger, text, util
 from molecule.console import should_do_markup
+from molecule.scenario import Scenario
+
+
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
+    from molecule.scenario import Scenario
 
 LOG = logging.getLogger(__name__)
 MOLECULE_GLOB = os.environ.get("MOLECULE_GLOB", "molecule/*/molecule.yml")
 MOLECULE_DEFAULT_SCENARIO_NAME = "default"
 
 
-class Base(object, metaclass=abc.ABCMeta):
+class Base(metaclass=abc.ABCMeta):
     """An abstract base class used to define the command interface."""
 
-    def __init__(self, c: config.Config):
-        """
-        Initialize code for all command classes.
+    def __init__(self, c: config.Config) -> None:
+        """Initialize code for all command classes.
 
-        :param c: An instance of a Molecule config.
-        :returns: None
+        Args:
+            c: An instance of a Molecule config.
+
+        Returns:
+            None
         """
         self._config = c
         self._setup()
@@ -56,26 +74,33 @@ class Base(object, metaclass=abc.ABCMeta):
         """Decorate execute from all subclasses."""
         super().__init_subclass__()
         for wrapper in logger.get_section_loggers():
-            setattr(cls, "execute", wrapper(cls.execute))
+            cls.execute = wrapper(cls.execute)  # type: ignore  # noqa: PGH003
 
     @abc.abstractmethod
-    def execute(self, action_args=None):  # pragma: no cover
-        pass
+    def execute(
+        self: Base,
+        action_args: list[str] | None = None,
+    ) -> None:  # pragma: no cover
+        """Abstract method to execute the command.
+
+        Args:
+            action_args: An optional list of arguments to pass to the action.
+        """
 
     def _setup(self) -> None:
-        """
-        Prepare Molecule's provisioner and returns None.
-
-        :return: None
-        """
+        """Prepare Molecule's provisioner and returns None."""
         self._config.write()
         self._config.provisioner.write_config()
         self._config.provisioner.manage_inventory()
 
 
-def execute_cmdline_scenarios(scenario_name, args, command_args, ansible_args=()):
-    """
-    Execute scenario sequences based on parsed command-line arguments.
+def execute_cmdline_scenarios(
+    scenario_name: str | None,
+    args: dict[str, Any],
+    command_args: dict[str, Any],
+    ansible_args: tuple[()] | tuple[str] = (),
+) -> None:
+    """Execute scenario sequences based on parsed command-line arguments.
 
     This is useful for subcommands that run scenario sequences, which
     excludes subcommands such as ``list``, ``login``, and ``matrix``.
@@ -83,18 +108,18 @@ def execute_cmdline_scenarios(scenario_name, args, command_args, ansible_args=()
     ``args`` and ``command_args`` are combined using :func:`get_configs`
     to generate the scenario(s) configuration.
 
-    :param scenario_name: Name of scenario to run, or ``None`` to run all.
-    :param args: ``args`` dict from ``click`` command context
-    :param command_args: dict of command arguments, including the target
-                         subcommand to execute
-    :returns: None
-
+    Args:
+        scenario_name: Name of scenario to run, or ``None`` to run all.
+        args: ``args`` dict from ``click`` command context
+        command_args: dict of command arguments, including the target
+        ansible_args: Optional tuple of arguments to pass to the `ansible-playbook` command
     """
     glob_str = MOLECULE_GLOB
     if scenario_name:
         glob_str = glob_str.replace("*", scenario_name)
     scenarios = molecule.scenarios.Scenarios(
-        get_configs(args, command_args, ansible_args, glob_str), scenario_name
+        get_configs(args, command_args, ansible_args, glob_str),
+        scenario_name,
     )
 
     if scenario_name and scenarios:
@@ -105,12 +130,12 @@ def execute_cmdline_scenarios(scenario_name, args, command_args, ansible_args=()
         )
 
     for scenario in scenarios:
-
         if scenario.config.config["prerun"]:
             role_name_check = scenario.config.config["role_name_check"]
             LOG.info("Performing prerun with role_name_check=%s...", role_name_check)
             scenario.config.runtime.prepare_environment(
-                install_local=True, role_name_check=role_name_check
+                install_local=True,
+                role_name_check=role_name_check,
             )
 
         if command_args.get("subcommand") == "reset":
@@ -133,79 +158,117 @@ def execute_cmdline_scenarios(scenario_name, args, command_args, ansible_args=()
                 # always prune ephemeral dir if destroying on failure
                 scenario.prune()
                 if scenario.config.is_parallel:
-                    scenario._remove_scenario_state_directory()
+                    scenario._remove_scenario_state_directory()  # noqa: SLF001
                 util.sysexit()
             else:
                 raise
 
 
-def execute_subcommand(config, subcommand_and_args):
-    """Execute subcommand."""
+def execute_subcommand(
+    current_config: config.Config,
+    subcommand_and_args: str,
+) -> Any:  # noqa: ANN401
+    """Execute subcommand.
+
+    Args:
+        current_config: An instance of a Molecule config.
+        subcommand_and_args: A string representing the subcommand and arguments.
+    """
     (subcommand, *args) = subcommand_and_args.split(" ")
     command_module = getattr(molecule.command, subcommand)
-    command = getattr(command_module, text.camelize(subcommand))
+    command = getattr(command_module, text.camelize(subcommand))  # type: ignore[no-untyped-call]
 
     # knowledge of the current action is used by some provisioners
     # to ensure they behave correctly during certain sequence steps,
     # particularly the setting of ansible options in create/destroy,
     # and is also used for reporting in execute_cmdline_scenarios
-    config.action = subcommand
+    current_config.action = subcommand
 
-    return command(config).execute(args)
+    return command(current_config).execute(args)
 
 
-def execute_scenario(scenario):
-    """
-    Execute each command in the given scenario's configured sequence.
+def execute_scenario(scenario: Scenario) -> None:
+    """Execute each command in the given scenario's configured sequence.
 
-    :param scenario: The scenario to execute.
-    :returns: None
+    Args:
+        scenario: The scenario to execute.
     """
     for action in scenario.sequence:
         execute_subcommand(scenario.config, action)
 
-    if (
-        "destroy" in scenario.sequence
-        and scenario.config.command_args.get("destroy") != "never"
-    ):
+    if "destroy" in scenario.sequence and scenario.config.command_args.get("destroy") != "never":
         scenario.prune()
 
         if scenario.config.is_parallel:
-            scenario._remove_scenario_state_directory()
+            scenario._remove_scenario_state_directory()  # noqa: SLF001
 
 
-def get_configs(args, command_args, ansible_args=(), glob_str=MOLECULE_GLOB):
+def filter_ignored_scenarios(scenario_paths) -> list[str]:  # type: ignore[no-untyped-def]  # noqa: ANN001, D103
+    command = ["git", "check-ignore", *scenario_paths]
+
+    with contextlib.suppress(subprocess.CalledProcessError, FileNotFoundError):
+        proc = subprocess.run(
+            args=command,
+            capture_output=True,
+            check=True,
+            text=True,
+            shell=False,
+        )
+
+    try:
+        ignored = proc.stdout.splitlines()
+        paths = [candidate for candidate in scenario_paths if str(candidate) not in ignored]
+    except NameError:
+        paths = scenario_paths
+
+    return paths
+
+
+def get_configs(
+    args: dict[str, Any],
+    command_args: dict[str, Any],
+    ansible_args: tuple[str, ...] = (),
+    glob_str: str = MOLECULE_GLOB,
+) -> list[config.Config]:
+    """Glob the current directory for Molecule config files.
+
+    Instantiate config objects, and returns a list.
+
+    Args:
+        args: A dict of options, arguments and commands from the CLI.
+        command_args: A dict of options passed to the subcommand from the CLI.
+        ansible_args: An optional tuple of arguments provided to the `ansible-playbook` command.
+        glob_str: A string representing the glob used to find Molecule config files.
+
+    Returns:
+        A list of Config objects.
     """
-    Glob the current directory for Molecule config files, instantiate config \
-    objects, and returns a list.
+    scenario_paths = glob.glob(
+        glob_str,
+        flags=wcmatch.pathlib.GLOBSTAR | wcmatch.pathlib.BRACE | wcmatch.pathlib.DOTGLOB,
+    )
 
-    :param args: A dict of options, arguments and commands from the CLI.
-    :param command_args: A dict of options passed to the subcommand from
-     the CLI.
-    :param ansible_args: An optional tuple of arguments provided to the
-     `ansible-playbook` command.
-    :return: list
-    """
+    scenario_paths = filter_ignored_scenarios(scenario_paths)
     configs = [
         config.Config(
-            molecule_file=util.abs_path(c),
+            molecule_file=util.abs_path(c),  # type: ignore[arg-type]
             args=args,
             command_args=command_args,
             ansible_args=ansible_args,
         )
-        for c in glob.glob(glob_str)
+        for c in scenario_paths
     ]
     _verify_configs(configs, glob_str)
 
     return configs
 
 
-def _verify_configs(configs, glob_str=MOLECULE_GLOB):
-    """
-    Verify a Molecule config was found and returns None.
+def _verify_configs(configs: list[config.Config], glob_str: str = MOLECULE_GLOB) -> None:
+    """Verify a Molecule config was found and returns None.
 
-    :param configs: A list containing absolute paths to Molecule config files.
-    :return: None
+    Args:
+        configs: A list containing absolute paths to Molecule config files.
+        glob_str: A string representing the glob used to find Molecule config files.
     """
     if configs:
         scenario_names = [c.scenario.name for c in configs]
@@ -219,11 +282,19 @@ def _verify_configs(configs, glob_str=MOLECULE_GLOB):
         util.sysexit_with_message(msg)
 
 
-def _get_subcommand(string):
+def _get_subcommand(string: str) -> str:
+    """Return the subcommand from a string.
+
+    Args:
+        string: A string containing a subcommand.
+
+    Returns:
+        A string representing the subcommand.
+    """
     return string.split(".")[-1]
 
 
-def click_group_ex():
+def click_group_ex():  # type: ignore[no-untyped-def]  # noqa: ANN201
     """Return extended version of click.group()."""
     # Color coding used to group command types, documented only here as we may
     # decide to change them later.
@@ -234,7 +305,7 @@ def click_group_ex():
         cls=HelpColorsGroup,
         # Workaround to disable click help line truncation to ~80 chars
         # https://github.com/pallets/click/issues/486
-        context_settings=dict(max_content_width=9999, color=should_do_markup()),
+        context_settings={"max_content_width": 9999, "color": should_do_markup()},
         help_headers_color="yellow",
         help_options_color="green",
         help_options_custom_colors={
@@ -252,12 +323,14 @@ def click_group_ex():
 
 def click_command_ex() -> Callable[[Callable[..., Any]], click.Command]:
     """Return extended version of click.command()."""
-    return click.command(  # type: ignore
-        cls=HelpColorsCommand, help_headers_color="yellow", help_options_color="green"
+    return click.command(
+        cls=HelpColorsCommand,
+        help_headers_color="yellow",
+        help_options_color="green",
     )
 
 
-def result_callback(*args, **kwargs):
+def result_callback(*args, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN002, ANN003, ANN201, ARG001
     """Click natural exit callback."""
     # We want to be used we run out custom exit code, regardless if run was
     # a success or failure.
